/*
 * comm.c
 *
 * Created: 07.03.2018 13:19:25
 *  Author: dsosnowski
 */ 


#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "serial.h"
#include "comm.h"
#include "lcd.h"


struct {
	uint8_t code;
	uint8_t len;
	uint8_t *offset;
} comm;


//------------------------------------------------------
uint8_t CommPrepare(void){
	uint8_t result = 0;
	uint8_t *src;
	
	if (SerialCheck() > 0){
		src = SerialGet();		
		src++;
		if (*src == 'L'){
			src++;
			if (*src == 'C'){
				src++;
				if (*src == 'D'){
					result = 1;
				}			
			}
		}
			
		if (result > 0){
			if (SerialLenGet() >= 6){
				src++;
				comm.code = *src;
				src++;
				comm.offset = src;
				comm.len = (SerialLenGet() - 6);
			} else {
				SerialClear();
			}
		} else {
			SerialClear();
		}
	}
	return comm.code;
}

//------------------------------------------------------
uint8_t CommCheck(void){
	if (comm.code > 0) return 1;
	return 0;
}

//------------------------------------------------------
void CommExe(void){

	uint8_t x = 0;	
	uint8_t y = 0;
	
	switch (comm.code){
		case SERIAL_COMM_WRITE:
			for(x = 0; x < comm.len; x++){
				LCDData(*comm.offset);
				comm.offset++;
			}
		break;
		case SERIAL_COMM_SET:
			y = CommValueWork(comm.offset);
			comm.offset = CommCommaWork(comm.offset);
			x = CommValueWork(comm.offset);
			if (y > 2) y = 2;
			if (y < 1) y = 1;
			if (x > 16) x = 16;
			if (x < 1) x = 1;
			LCDLocate((y-1),(x-1));					
		break;
		case SERIAL_COMM_CLR:
			LCDClear();
			LCDLocate(0,0);	
		break;
		case SERIAL_COMM_SPEC:
			x = CommValueWork(comm.offset);
			comm.offset = CommCommaWork(comm.offset);
			LCDSetSpecChar(x,comm.offset);
			LCDLocate(0,0);
		break;
	}
}

//------------------------------------------------------
void CommClear(void){
	comm.code = 0;
	comm.offset = 0;
	comm.len = 0;
	SerialClear();
}

//------------------------------------------------------
uint8_t CommValueWork(uint8_t *src){
	uint8_t result = 0;
	
	if (*src == ' ')src++;

	if ((*src >= '0')&&(*src <= '9')){
		result += ((*src) - '0');

		src++;
		if ((*src >= '0')&&(*src <= '9')){
			result *= 10;
			result += ((*src) - '0');

			src++;
			if ((*src >= '0')&&(*src <= '9')){
				result *= 10;
				result += ((*src) - '0');
			}
		}
	} else {
		if (*src == 'x'){
			src++;

			if ((*src >= '0')&&(*src <= '9')){
				result += ((*src) - '0');
				src++;
			} else if ((*src >= 'A')&&(*src <= 'F')){
				result += (((*src) - 'A') + 10);
				src++;
			} 

			if ((*src >= '0')&&(*src <= '9')){
				result <<= 4;
				result += ((*src) - '0');
			} else if ((*src >= 'A')&&(*src <= 'F')){
				result <<= 4;
				result += (((*src) - 'A') + 10);
			}
		}
	}
	
	return result;
}

//------------------------------------------------------
uint8_t *CommCommaWork(uint8_t *src){
	while ((*src != ',') && (*src != '.') && (*src != SERIAL_POSTFIX)){
		src++;
	}
	src++;
	return src;
}