/*
 * serial.c
 *
 * Created: 07.03.2018 12:55:45
 *  Author: dsosnowski
 */ 


#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "serial.h"

#define SERIAL_BUFF					48

struct {
	volatile uint8_t counter;
	volatile uint8_t status;
	uint8_t buff[SERIAL_BUFF];	
} serial;


#define SERIAL_RXD_PIN			2


//----------------------------------------------------------------------
void SerialInit(void){
	
	DDRD &= ~(1<<SERIAL_RXD_PIN);
	PORTD |= (1<<SERIAL_RXD_PIN);
	
	//config USART
	UCSRB = (1<<RXCIE | 1<<RXEN);				//RX int en
	UCSRC = (1<<UCSZ1 | 1<<UCSZ0);				//8 data, 1 stop, parity none
	UBRRL = 35;									//UBRRL = 35 - 19.2k

	sei();		
}

//-------------------------------------
ISR(USART_RX_vect){
	char temp = UDR;
	if (temp == SERIAL_KEY){
		if (serial.status == 0){
			serial.status = 1;
			serial.counter = 0;
		}
	}
	
	if (serial.status == 1){
		if (serial.counter < SERIAL_BUFF){
			serial.buff[serial.counter] = temp;
			serial.counter++;
		}
	}
	
	if (temp == SERIAL_POSTFIX){
		if (serial.status == 1){
			serial.status = 2;
		}
	}	
}

//----------------------------------------------------------------------
uint8_t SerialCheck(void){
	if (serial.status == 2) return 1;
	return 0;
}

//----------------------------------------------------------------------
uint8_t SerialLenGet(void){
	return serial.counter;
}

//----------------------------------------------------------------------
uint8_t *SerialGet(void){
	return serial.buff;
}

//----------------------------------------------------------------------
void SerialClear(void){
	serial.buff[0] = 0;
	serial.counter = 0;
	serial.status = 0;
}

